package org.seasar.doma.internal.apt;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.seasar.doma.internal.Constants.EXTERNAL_DOMAIN_TYPE_ARRAY_SUFFIX;

import org.junit.jupiter.api.extension.ExtensionContext;
import org.junit.jupiter.api.extension.ParameterContext;
import org.junit.jupiter.api.extension.ParameterResolutionException;
import org.junit.jupiter.api.extension.ParameterResolver;
import org.opentest4j.AssertionFailedError;
import org.seasar.doma.Dao;
import org.seasar.doma.Domain;
import org.seasar.doma.Embeddable;
import org.seasar.doma.Entity;
import org.seasar.doma.internal.ClassNames;
import org.seasar.doma.internal.EmbeddableDesc;
import org.seasar.doma.internal.EntityDesc;

public class GeneratedClassNameParameterResolver implements ParameterResolver {

  private final Class<?> clazz;

  private final boolean isExternalDomain;

  public GeneratedClassNameParameterResolver(Class<?> clazz) {
    assertNotNull(clazz);
    this.clazz = clazz;
    this.isExternalDomain = false;
  }

  public GeneratedClassNameParameterResolver(Class<?> clazz, boolean isExternalDomain) {
    assertNotNull(clazz);
    this.clazz = clazz;
    this.isExternalDomain = true;
  }

  @Override
  public boolean supportsParameter(
      ParameterContext parameterContext, ExtensionContext extensionContext)
      throws ParameterResolutionException {
    return parameterContext.getParameter().getType().equals(String.class);
  }

  @Override
  public Object resolveParameter(
      ParameterContext parameterContext, ExtensionContext extensionContext)
      throws ParameterResolutionException {
    if (isExternalDomain) {
      String name =
          clazz.isArray()
              ? clazz.getComponentType().getName() + EXTERNAL_DOMAIN_TYPE_ARRAY_SUFFIX
              : clazz.getName();
      return ClassNames.newExternalDomainTypeClassName(name).toString();
    }
    if (clazz.isAnnotationPresent(Dao.class)) {
      return clazz.getName() + Options.Constants.DEFAULT_DAO_SUFFIX;
    }
    if (clazz.isAnnotationPresent(Entity.class)) {
      return ClassNames.newEntityTypeClassName(clazz.getName()).toString();
    }
    if (clazz.isAnnotationPresent(Embeddable.class)) {
      return ClassNames.newEmbeddableTypeClassName(clazz.getName()).toString();
    }
    if (clazz.isAnnotationPresent(Domain.class)) {
      return ClassNames.newDomainTypeClassName(clazz.getName()).toString();
    }
    if (clazz.isAnnotationPresent(EntityDesc.class)) {
      EntityDesc entityDesc = clazz.getAnnotation(EntityDesc.class);
      return entityDesc.value().getName() + "_";
    }
    if (clazz.isAnnotationPresent(EmbeddableDesc.class)) {
      EmbeddableDesc embeddableDesc = clazz.getAnnotation(EmbeddableDesc.class);
      return embeddableDesc.value().getName() + "_";
    }
    throw new AssertionFailedError("annotation not found.");
  }
}
